@@ -2,67 +2,34 @@
     This file is part of the KDE libraries
 
     SPDX-FileCopyrightText: 2009 David Faure <faure@kde.org>
+    SPDX-FileCopyrightText: 2023 Harald Sitter <sitter@kde.org>
 
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include <kdirwatch.h>
+#include <kdirwatch_p.h>
 
-#include <QDir>
 #include <QDebug>
+#include <QDir>
 #include <QFileInfo>
+#include <QSignalSpy>
 #include <QTemporaryDir>
 #include <QTest>
-#include <QSignalSpy>
 #include <QThread>
 #include <sys/stat.h>
 #ifdef Q_OS_UNIX
 #include <unistd.h> // ::link()
 #endif
 
-#include "config-tests.h"
+#include "kcoreaddons_debug.h"
+#include "kdirwatch_test_utils.h"
+
+using namespace KDirWatchTestUtils;
 
 // Debugging notes: to see which inotify signals are emitted, either set s_verboseDebug=true
 // at the top of kdirwatch.cpp, or use the command-line tool "inotifywait -m /path"
 
-// Note that kdirlistertest and kdirmodeltest also exercise KDirWatch quite a lot.
-
-static const char *methodToString(KDirWatch::Method method)
-{
-    switch (method) {
-    case KDirWatch::FAM:
-        return "Fam";
-    case KDirWatch::INotify:
-        return "INotify";
-    case KDirWatch::Stat:
-        return "Stat";
-    case KDirWatch::QFSWatch:
-        return "QFSWatch";
-    }
-    return "ERROR!";
-}
-
-class StaticObject
-{
-public:
-    KDirWatch m_dirWatch;
-};
-Q_GLOBAL_STATIC(StaticObject, s_staticObject)
-
-class StaticObjectUsingSelf // like KSambaShare does, bug 353080
-{
-public:
-    StaticObjectUsingSelf() {
-        KDirWatch::self();
-    }
-    ~StaticObjectUsingSelf() {
-        if (KDirWatch::exists() && KDirWatch::self()->contains(QDir::homePath())) {
-            KDirWatch::self()->removeDir(QDir::homePath());
-        }
-    }
-};
-Q_GLOBAL_STATIC(StaticObjectUsingSelf, s_staticObjectUsingSelf)
-
 class KDirWatch_UnitTest : public QObject
 {
     Q_OBJECT
@@ -77,8 +44,8 @@ public:
         m_path = m_tempDir.path() + QLatin1Char('/');
         KDirWatch *dirW = &s_staticObject()->m_dirWatch;
         m_stat = dirW->internalMethod() == KDirWatch::Stat;
-        m_slow = (dirW->internalMethod() == KDirWatch::FAM || m_stat);
-        qDebug() << "Using method" << methodToString(dirW->internalMethod());
+        m_slow = m_stat;
+        qCDebug(KCOREADDONS_DEBUG) << "Using method" << methodToString(dirW->internalMethod());
     }
 
 private Q_SLOTS: // test methods
@@ -108,29 +75,27 @@ private Q_SLOTS: // test methods
     void nestedEventLoop();
     void testHardlinkChange();
     void stopAndRestart();
-    void benchCreateTree();
-    void benchCreateWatcher();
-    void benchNotifyWatcher();
     void testRefcounting();
+    void testRelativeRefcounting();
+    void testMoveToThread();
 
 protected Q_SLOTS: // internal slots
     void nestedEventLoopSlot();
 
 private:
-    void waitUntilMTimeChange(const QString &path);
-    void waitUntilNewSecond();
-    void waitUntilAfter(const QDateTime &ctime);
     QList<QVariantList> waitForDirtySignal(KDirWatch &watch, int expected);
     QList<QVariantList> waitForDeletedSignal(KDirWatch &watch, int expected);
     bool waitForOneSignal(KDirWatch &watch, const char *sig, const QString &path);
     bool waitForRecreationSignal(KDirWatch &watch, const QString &path);
     bool verifySignalPath(QSignalSpy &spy, const char *sig, const QString &expectedPath);
-    void createFile(const QString &path);
     QString createFile(int num);
+    void createFile(const QString &file)
+    {
+        KDirWatchTestUtils::createFile(file, m_slow);
+    }
     void removeFile(int num);
     void appendToFile(const QString &path);
     void appendToFile(int num);
-    int createDirectoryTree(const QString &path, int depth = 4);
 
     QTemporaryDir m_tempDir;
     QString m_path;
@@ -140,34 +105,13 @@ private:
 
 QTEST_MAIN(KDirWatch_UnitTest)
 
-// Just to make the inotify packets bigger
-static const char s_filePrefix[] = "This_is_a_test_file_";
-
 static const int s_maxTries = 50;
 
-// helper method: create a file
-void KDirWatch_UnitTest::createFile(const QString &path)
-{
-    QFile file(path);
-    QVERIFY(file.open(QIODevice::WriteOnly));
-#ifdef Q_OS_FREEBSD
-    // FreeBSD has inotify implemented as user-space library over native kevent API.
-    // When using it, one has to open() a file to start watching it, so workaround
-    // test breakage by giving inotify time to react to file creation.
-    // Full context: https://github.com/libinotify-kqueue/libinotify-kqueue/issues/10
-    if(!m_slow)
-        QThread::msleep(1);
-#endif
-    file.write(QByteArray("foo"));
-    file.close();
-    //qDebug() << path;
-}
-
 // helper method: create a file (identified by number)
 QString KDirWatch_UnitTest::createFile(int num)
 {
     const QString fileName = QLatin1String(s_filePrefix) + QString::number(num);
-    createFile(m_path + fileName);
+    KDirWatchTestUtils::createFile(m_path + fileName, m_slow);
     return m_path + fileName;
 }
 
@@ -178,87 +122,16 @@ void KDirWatch_UnitTest::removeFile(int num)
     QFile::remove(m_path + fileName);
 }
 
-int KDirWatch_UnitTest::createDirectoryTree(const QString& basePath, int depth)
-{
-    int filesCreated = 0;
-
-    const int numFiles = 10;
-    for (int i = 0; i < numFiles; ++i) {
-        createFile(basePath + QLatin1Char('/') + QLatin1String(s_filePrefix) + QString::number(i));
-        ++filesCreated;
-    }
-
-    if (depth <= 0) {
-        return filesCreated;
-    }
-
-    const int numFolders = 5;
-    for (int i = 0; i < numFolders; ++i) {
-        const QString childPath = basePath + QLatin1String("/subdir") + QString::number(i);
-        QDir().mkdir(childPath);
-        filesCreated += createDirectoryTree(childPath, depth - 1);
-    }
-
-    return filesCreated;
-}
-
-void KDirWatch_UnitTest::waitUntilMTimeChange(const QString &path)
-{
-    // Wait until the current second is more than the file's mtime
-    // otherwise this change will go unnoticed
-
-    QFileInfo fi(path);
-    QVERIFY(fi.exists());
-    const QDateTime ctime = fi.lastModified();
-    waitUntilAfter(ctime);
-
-}
-
-void KDirWatch_UnitTest::waitUntilNewSecond()
-{
-    QDateTime now = QDateTime::currentDateTime();
-    waitUntilAfter(now);
-}
-
-void KDirWatch_UnitTest::waitUntilAfter(const QDateTime &ctime)
-{
-    int totalWait = 0;
-    QDateTime now;
-    Q_FOREVER {
-        now = QDateTime::currentDateTime();
-        if (now.toMSecsSinceEpoch() / 1000 == ctime.toMSecsSinceEpoch() / 1000)   // truncate milliseconds
-        {
-            totalWait += 50;
-            QTest::qWait(50);
-        } else {
-            QVERIFY(now > ctime); // can't go back in time ;)
-            QTest::qWait(50); // be safe
-            break;
-        }
-    }
-    //if (totalWait > 0)
-    qDebug() << "Waited" << totalWait << "ms so that now" << now.toString(Qt::ISODate) << "is >" << ctime.toString(Qt::ISODate);
-}
-
 // helper method: modifies a file
 void KDirWatch_UnitTest::appendToFile(const QString &path)
 {
     QVERIFY(QFile::exists(path));
     waitUntilMTimeChange(path);
-    //const QString directory = QDir::cleanPath(path+"/..");
-    //waitUntilMTimeChange(directory);
 
     QFile file(path);
     QVERIFY(file.open(QIODevice::Append | QIODevice::WriteOnly));
     file.write(QByteArray("foobar"));
     file.close();
-#if 0
-        QFileInfo fi(path);
-        QVERIFY(fi.exists());
-        qDebug() << "After append: file ctime=" << fi.lastModified().toString(Qt::ISODate);
-        QVERIFY(fi.exists());
-        qDebug() << "After append: metadataChangeTime" << fi.metadataChangeTime().toString(Qt::ISODate);
-#endif
 }
 
 // helper method: modifies a file (identified by number)
@@ -280,17 +153,17 @@ static QString removeTrailingSlash(const QString &path)
 // helper method
 QList<QVariantList> KDirWatch_UnitTest::waitForDirtySignal(KDirWatch &watch, int expected)
 {
-    QSignalSpy spyDirty(&watch, SIGNAL(dirty(QString)));
+    QSignalSpy spyDirty(&watch, &KDirWatch::dirty);
     int numTries = 0;
     // Give time for KDirWatch to notify us
     while (spyDirty.count() < expected) {
         if (++numTries > s_maxTries) {
             qWarning() << "Timeout waiting for KDirWatch. Got" << spyDirty.count() << "dirty() signals, expected" << expected;
-            return std::move(spyDirty);
+            return spyDirty;
         }
         spyDirty.wait(50);
     }
-    return std::move(spyDirty);
+    return spyDirty;
 }
 
 bool KDirWatch_UnitTest::waitForOneSignal(KDirWatch &watch, const char *sig, const QString &path)
@@ -313,72 +186,72 @@ bool KDirWatch_UnitTest::waitForOneSignal(KDirWatch &watch, const char *sig, con
 
 bool KDirWatch_UnitTest::verifySignalPath(QSignalSpy &spy, const char *sig, const QString &expectedPath)
 {
-  for (int i = 0; i < spy.count(); ++i) {
-    const QString got = spy[i][0].toString();
-    if (got == expectedPath) {
-      return true;
+    for (int i = 0; i < spy.count(); ++i) {
+        const QString got = spy[i][0].toString();
+        if (got == expectedPath) {
+            return true;
+        }
+        if (got.startsWith(expectedPath + QLatin1Char('/'))) {
+            qCDebug(KCOREADDONS_DEBUG) << "Ignoring (inotify) notification of" << (sig + 1) << '(' << got << ')';
+            continue;
+        }
+        qWarning() << "Expected" << sig << '(' << expectedPath << ')' << "but got" << sig << '(' << got << ')';
+        return false;
     }
-    if (got.startsWith(expectedPath + QLatin1Char('/'))) {
-      qDebug() << "Ignoring (inotify) notification of" << (sig + 1) << '(' << got << ')';
-      continue;
-    }
-    qWarning() << "Expected" << sig << '(' << expectedPath << ')' << "but got" << sig << '(' << got << ')';
     return false;
-  }
-  return false;
 }
 
 bool KDirWatch_UnitTest::waitForRecreationSignal(KDirWatch &watch, const QString &path)
 {
-  // When watching for a deleted + created signal pair, the two might come so close that
-  // using waitForOneSignal will miss the created signal.  This function monitors both all
-  // the time to ensure both are received.
-  //
-  // In addition, it allows dirty() to be emitted (for that same path) as an alternative
+    // When watching for a deleted + created signal pair, the two might come so close that
+    // using waitForOneSignal will miss the created signal.  This function monitors both all
+    // the time to ensure both are received.
+    //
+    // In addition, it allows dirty() to be emitted (for that same path) as an alternative
 
-  const QString expectedPath = removeTrailingSlash(path);
-  QSignalSpy spyDirty(&watch, &KDirWatch::dirty);
-  QSignalSpy spyDeleted(&watch, &KDirWatch::deleted);
-  QSignalSpy spyCreated(&watch, &KDirWatch::created);
+    const QString expectedPath = removeTrailingSlash(path);
+    QSignalSpy spyDirty(&watch, &KDirWatch::dirty);
+    QSignalSpy spyDeleted(&watch, &KDirWatch::deleted);
+    QSignalSpy spyCreated(&watch, &KDirWatch::created);
 
-  int numTries = 0;
-  while (spyDeleted.isEmpty() && spyDirty.isEmpty()) {
-      if (++numTries > s_maxTries) {
-          return false;
-      }
-      spyDeleted.wait(50);
-      while (!spyDirty.isEmpty()) {
-          if (spyDirty.at(0).at(0).toString() != expectedPath) { // unrelated
-              spyDirty.removeFirst();
-          }
-      }
-  }
-  if (!spyDirty.isEmpty()) {
-      return true;
-  }
+    int numTries = 0;
+    while (spyDeleted.isEmpty() && spyDirty.isEmpty()) {
+        if (++numTries > s_maxTries) {
+            return false;
+        }
+        spyDeleted.wait(50);
+        while (!spyDirty.isEmpty()) {
+            if (spyDirty.at(0).at(0).toString() != expectedPath) { // unrelated
+                spyDirty.removeFirst();
+            }
+        }
+    }
+    if (!spyDirty.isEmpty()) {
+        return true;
+    }
 
-  // Don't bother waiting for the created signal if the signal spy already received a signal.
-  if(spyCreated.isEmpty() && !spyCreated.wait(50 * s_maxTries)) {
-    qWarning() << "Timeout waiting for KDirWatch signal created(QString) (" << path << ")";
-    return false;
-  }
+    // Don't bother waiting for the created signal if the signal spy already received a signal.
+    if (spyCreated.isEmpty() && !spyCreated.wait(50 * s_maxTries)) {
+        qWarning() << "Timeout waiting for KDirWatch signal created(QString) (" << path << ")";
+        return false;
+    }
 
-  return verifySignalPath(spyDeleted, "deleted(QString)", expectedPath) && verifySignalPath(spyCreated, "created(QString)", expectedPath);
+    return verifySignalPath(spyDeleted, "deleted(QString)", expectedPath) && verifySignalPath(spyCreated, "created(QString)", expectedPath);
 }
 
 QList<QVariantList> KDirWatch_UnitTest::waitForDeletedSignal(KDirWatch &watch, int expected)
 {
-    QSignalSpy spyDeleted(&watch, SIGNAL(created(QString)));
+    QSignalSpy spyDeleted(&watch, &KDirWatch::created);
     int numTries = 0;
     // Give time for KDirWatch to notify us
     while (spyDeleted.count() < expected) {
         if (++numTries > s_maxTries) {
             qWarning() << "Timeout waiting for KDirWatch. Got" << spyDeleted.count() << "deleted() signals, expected" << expected;
-            return std::move(spyDeleted);
+            return spyDeleted;
         }
         spyDeleted.wait(50);
     }
-    return std::move(spyDeleted);
+    return spyDeleted;
 }
 
 void KDirWatch_UnitTest::touchOneFile() // watch a dir, create a file in it
@@ -390,7 +263,7 @@ void KDirWatch_UnitTest::touchOneFile() // watch a dir, create a file in it
     waitUntilMTimeChange(m_path);
 
     // dirty(the directory) should be emitted.
-    QSignalSpy spyCreated(&watch, SIGNAL(created(QString)));
+    QSignalSpy spyCreated(&watch, &KDirWatch::created);
     createFile(0);
     QVERIFY(waitForOneSignal(watch, SIGNAL(dirty(QString)), m_path));
     QCOMPARE(spyCreated.count(), 0); // "This is not emitted when creating a file is created in a watched directory."
@@ -414,7 +287,6 @@ void KDirWatch_UnitTest::touch1000Files()
     QList<QVariantList> spy = waitForDirtySignal(watch, fileCount);
     if (watch.internalMethod() == KDirWatch::INotify) {
         QVERIFY(spy.count() >= fileCount);
-        qDebug() << spy.count();
     } else {
         // More stupid backends just see one mtime change on the directory
         QVERIFY(spy.count() >= 1);
@@ -446,7 +318,7 @@ void KDirWatch_UnitTest::removeAndReAdd()
     watch.addDir(QStringLiteral(":/kio5/newfile-templates"));
     watch.startScan();
     if (watch.internalMethod() != KDirWatch::INotify) {
-        waitUntilNewSecond();    // necessary for mtime checks in scanEntry
+        waitUntilNewSecond(); // necessary for mtime checks in scanEntry
     }
     createFile(0);
     QVERIFY(waitForOneSignal(watch, SIGNAL(dirty(QString)), m_path));
@@ -455,7 +327,7 @@ void KDirWatch_UnitTest::removeAndReAdd()
     watch.removeDir(m_path);
     watch.addDir(m_path);
     if (watch.internalMethod() != KDirWatch::INotify) {
-        waitUntilMTimeChange(m_path);    // necessary for FAM and QFSWatcher
+        waitUntilMTimeChange(m_path); // necessary for QFSWatcher
     }
     createFile(1);
     QVERIFY(waitForOneSignal(watch, SIGNAL(dirty(QString)), m_path));
@@ -475,12 +347,12 @@ void KDirWatch_UnitTest::watchNonExistent()
     }
 
     // Now create it, KDirWatch should emit created()
-    qDebug() << "Creating" << subdir;
+    qCDebug(KCOREADDONS_DEBUG) << "Creating" << subdir;
     QDir().mkdir(subdir);
 
     QVERIFY(waitForOneSignal(watch, SIGNAL(created(QString)), subdir));
 
-    KDirWatch::statistics();
+    qCDebug(KCOREADDONS_DEBUG) << &watch;
 
     // Play with addDir/removeDir, just for fun
     watch.addDir(subdir);
@@ -494,11 +366,11 @@ void KDirWatch_UnitTest::watchNonExistent()
     watch.addFile(file1); // doesn't exist yet
     watch.removeFile(file1); // forget it again
 
-    KDirWatch::statistics();
+    qCDebug(KCOREADDONS_DEBUG) << &watch;
 
     QVERIFY(!QFile::exists(file));
     // Now create it, KDirWatch should emit created
-    qDebug() << "Creating" << file;
+    qCDebug(KCOREADDONS_DEBUG) << "Creating" << file;
     createFile(file);
     QVERIFY(waitForOneSignal(watch, SIGNAL(created(QString)), file));
 
@@ -514,8 +386,8 @@ void KDirWatch_UnitTest::watchNonExistentWithSingleton()
 {
     const QString file = QLatin1String("/root/.ssh/authorized_keys");
     KDirWatch::self()->addFile(file);
-    // When running this test in KDIRWATCH_METHOD=QFSWatch, or when FAM is not available
-    // and we fallback to qfswatch when inotify fails above, we end up creating the fsWatch
+    // When running this test in KDIRWATCH_METHOD=QFSWatch, or when we fallback to qfswatch
+    // when inotify fails above, we end up creating the fsWatch
     // in the kdirwatch singleton. Bug 261541 discovered that Qt hanged when deleting fsWatch
     // once QCoreApp was gone, this is what this test is about.
 }
@@ -532,9 +404,9 @@ void KDirWatch_UnitTest::testDelete()
     KDirWatch watch;
     watch.addFile(file1);
 
-    KDirWatch::statistics();
+    qCDebug(KCOREADDONS_DEBUG) << &watch;
 
-    QSignalSpy spyDirty(&watch, SIGNAL(dirty(QString)));
+    QSignalSpy spyDirty(&watch, &KDirWatch::dirty);
     QFile::remove(file1);
     QVERIFY(waitForOneSignal(watch, SIGNAL(deleted(QString)), file1));
     QTest::qWait(40); // just in case delayed processing would emit it
@@ -555,18 +427,13 @@ void KDirWatch_UnitTest::testDeleteAndRecreateFile() // Useful for /etc/localtim
     KDirWatch watch;
     watch.addFile(file1);
 
-    //KDE_struct_stat stat_buf;
-    //QCOMPARE(KDE::stat(QFile::encodeName(file1), &stat_buf), 0);
-    //qDebug() << "initial inode" << stat_buf.st_ino;
-
     // Make sure this even works multiple times, as needed for ksycoca
     for (int i = 0; i < 5; ++i) {
-
         if (m_slow || watch.internalMethod() == KDirWatch::QFSWatch) {
             waitUntilNewSecond();
         }
 
-        qDebug() << "Attempt #" << (i+1) << "removing+recreating" << file1;
+        qCDebug(KCOREADDONS_DEBUG) << "Attempt #" << (i + 1) << "removing+recreating" << file1;
 
         // When watching for a deleted + created signal pair, the two might come so close that
         // using waitForOneSignal will miss the created signal.  This function monitors both all
@@ -584,8 +451,6 @@ void KDirWatch_UnitTest::testDeleteAndRecreateFile() // Useful for /etc/localtim
         // And recreate immediately, to try and fool KDirWatch with unchanged ctime/mtime ;)
         // (This emulates the /etc/localtime case)
         createFile(file1);
-        //QCOMPARE(KDE::stat(QFile::encodeName(file1), &stat_buf), 0);
-        //qDebug() << "new inode" << stat_buf.st_ino; // same!
 
         // THEN
         int numTries = 0;
@@ -672,10 +537,10 @@ void KDirWatch_UnitTest::testMoveTo()
     createFile(filetemp);
     QFile::remove(file1);
     QVERIFY(QFile::rename(filetemp, file1)); // overwrite file1 with the tempfile
-    qDebug() << "Overwrite file1 with tempfile";
+    qCDebug(KCOREADDONS_DEBUG) << "Overwrite file1 with tempfile";
 
-    QSignalSpy spyCreated(&watch, SIGNAL(created(QString)));
-    QSignalSpy spyDirty(&watch, SIGNAL(dirty(QString)));
+    QSignalSpy spyCreated(&watch, &KDirWatch::created);
+    QSignalSpy spyDirty(&watch, &KDirWatch::dirty);
     QVERIFY(waitForOneSignal(watch, SIGNAL(dirty(QString)), m_path));
 
     // Getting created() on an unwatched file is an inotify bonus, it's not part of the requirements.
@@ -691,11 +556,7 @@ void KDirWatch_UnitTest::testMoveTo()
     appendToFile(file1);
     QVERIFY(waitForOneSignal(watch, SIGNAL(dirty(QString)), file1));
 
-    //qDebug() << "after created";
-    //KDirWatch::statistics();
     watch.removeFile(file1); // now we remove it
-    //qDebug() << "after removeFile";
-    //KDirWatch::statistics();
 
     // Just touch another file to trigger a findSubEntry - this where the crash happened
     waitUntilMTimeChange(m_path);
@@ -725,8 +586,8 @@ void KDirWatch_UnitTest::nestedEventLoop() // #220153: watch two files, and modi
     appendToFile(file0);
 
     // use own spy, to connect it before nestedEventLoopSlot, otherwise it reverses order
-    QSignalSpy spyDirty(&watch, SIGNAL(dirty(QString)));
-    connect(&watch, SIGNAL(dirty(QString)), this, SLOT(nestedEventLoopSlot()));
+    QSignalSpy spyDirty(&watch, &KDirWatch::dirty);
+    connect(&watch, &KDirWatch::dirty, this, &KDirWatch_UnitTest::nestedEventLoopSlot);
     waitForDirtySignal(watch, 1);
     QVERIFY(spyDirty.count() >= 2);
     QCOMPARE(spyDirty[0][0].toString(), file0);
@@ -738,16 +599,14 @@ void KDirWatch_UnitTest::nestedEventLoopSlot()
     const KDirWatch *const_watch = qobject_cast<const KDirWatch *>(sender());
     KDirWatch *watch = const_cast<KDirWatch *>(const_watch);
     // let's not come in this slot again
-    disconnect(watch, SIGNAL(dirty(QString)), this, SLOT(nestedEventLoopSlot()));
+    disconnect(watch, &KDirWatch::dirty, this, &KDirWatch_UnitTest::nestedEventLoopSlot);
 
     const QString file1 = m_path + QLatin1String("nested_1");
     appendToFile(file1);
-    //qDebug() << "now waiting for signal";
     // The nested event processing here was from a messagebox in #220153
     QList<QVariantList> spy = waitForDirtySignal(*watch, 1);
     QVERIFY(spy.count() >= 1);
     QCOMPARE(spy[spy.count() - 1][0].toString(), file1);
-    //qDebug() << "done";
 
     // Now the user pressed reload...
     const QString file0 = m_path + QLatin1String("nested_0");
@@ -763,16 +622,13 @@ void KDirWatch_UnitTest::testHardlinkChange()
     // described on kde-core-devel (2009-07-03).
     // It shows that watching a specific file doesn't inform us that the file is
     // being recreated. Better watch the directory, for that.
-    // Well, it works with inotify (and fam - which uses inotify I guess?)
+    // Well, it works with inotify
 
     const QString existingFile = m_path + QLatin1String("ExistingFile");
     KDirWatch watch;
     watch.addFile(existingFile);
     watch.startScan();
 
-    //waitUntilMTimeChange(existingFile);
-    //waitUntilMTimeChange(m_path);
-
     QFile::remove(existingFile);
     const QString testFile = m_path + QLatin1String("TestFile");
     QVERIFY(::link(QFile::encodeName(testFile).constData(), QFile::encodeName(existingFile).constData()) == 0); // make ExistingFile "point" to TestFile
@@ -780,8 +636,6 @@ void KDirWatch_UnitTest::testHardlinkChange()
 
     QVERIFY(waitForRecreationSignal(watch, existingFile));
 
-    //KDirWatch::statistics();
-
     // The mtime of the existing file is the one of "TestFile", so it's old.
     // We won't detect the change then, if we use that as baseline for waiting.
     // We really need msec granularity, but that requires using statx which isn't available everywhere...
@@ -803,11 +657,11 @@ void KDirWatch_UnitTest::stopAndRestart()
 
     watch.stopDirScan(m_path);
 
-    //qDebug() << "create file 2 at" << QDateTime::currentDateTime().toMSecsSinceEpoch();
+    qCDebug(KCOREADDONS_DEBUG) << "create file 2 at" << QDateTime::currentDateTime().toMSecsSinceEpoch();
     const QString file2 = createFile(2);
-    QSignalSpy spyDirty(&watch, SIGNAL(dirty(QString)));
+    QSignalSpy spyDirty(&watch, &KDirWatch::dirty);
     QTest::qWait(200);
-    QCOMPARE(spyDirty.count(), 0);// suspended -> no signal
+    QCOMPARE(spyDirty.count(), 0); // suspended -> no signal
 
     watch.restartDirScan(m_path);
 
@@ -819,11 +673,11 @@ void KDirWatch_UnitTest::stopAndRestart()
     // comparison ("trust QFSW since the mtime of dirs isn't modified")
 #endif
 
-    KDirWatch::statistics();
+    qCDebug(KCOREADDONS_DEBUG) << &watch;
 
     waitUntilMTimeChange(m_path); // necessary for the mtime comparison in scanEntry
 
-    //qDebug() << "create file 3 at" << QDateTime::currentDateTime().toMSecsSinceEpoch();
+    qCDebug(KCOREADDONS_DEBUG) << "create file 3 at" << QDateTime::currentDateTime().toMSecsSinceEpoch();
     const QString file3 = createFile(3);
 #ifdef Q_OS_WIN
     if (watch.internalMethod() == KDirWatch::QFSWatch) {
@@ -836,54 +690,6 @@ void KDirWatch_UnitTest::stopAndRestart()
     QFile::remove(file3);
 }
 
-void KDirWatch_UnitTest::benchCreateTree()
-{
-#if !ENABLE_BENCHMARKS
-    QSKIP("Benchmarks are disabled in debug mode");
-#endif
-    QTemporaryDir dir;
-
-    QBENCHMARK {
-        createDirectoryTree(dir.path());
-    }
-}
-
-void KDirWatch_UnitTest::benchCreateWatcher()
-{
-#if !ENABLE_BENCHMARKS
-    QSKIP("Benchmarks are disabled in debug mode");
-#endif
-    QTemporaryDir dir;
-    createDirectoryTree(dir.path());
-
-    QBENCHMARK {
-        KDirWatch watch;
-        watch.addDir(dir.path(), KDirWatch::WatchSubDirs | KDirWatch:: WatchFiles);
-    }
-}
-
-void KDirWatch_UnitTest::benchNotifyWatcher()
-{
-#if !ENABLE_BENCHMARKS
-    QSKIP("Benchmarks are disabled in debug mode");
-#endif
-    QTemporaryDir dir;
-    // create the dir once upfront
-    auto numFiles = createDirectoryTree(dir.path());
-    waitUntilMTimeChange(dir.path());
-
-    KDirWatch watch;
-    watch.addDir(dir.path(), KDirWatch::WatchSubDirs | KDirWatch:: WatchFiles);
-
-    // now touch all the files repeatedly and wait for the dirty updates to come in
-    QSignalSpy spy(&watch, &KDirWatch::dirty);
-    QBENCHMARK {
-        createDirectoryTree(dir.path());
-        QTRY_COMPARE_WITH_TIMEOUT(spy.count(), numFiles, s_maxTries * 50 * 2);
-        spy.clear();
-    }
-}
-
 void KDirWatch_UnitTest::testRefcounting()
 {
 #if QT_CONFIG(cxx11_future)
@@ -906,4 +712,55 @@ void KDirWatch_UnitTest::testRefcounting()
 #endif
 }
 
+void KDirWatch_UnitTest::testRelativeRefcounting()
+{
+    // Relative files aren't supported but should still result in correct ref
+    // handling. Specifically when watch1 gets destroyed it should take all
+    // its entries with it regardless of whether the entry was well-formed
+    KDirWatch watch0;
+
+    if (watch0.internalMethod() != KDirWatch::INotify) {
+        // Only test on inotify. Otherwise Entry count expectations may diverge.
+        return;
+    }
+
+    const auto initialSize = watch0.d->m_mapEntries.size();
+    {
+        KDirWatch watch1;
+        watch1.addFile(QStringLiteral("AVeryRelativePath.txt"));
+        // NOTE: addFile actually adds two entires: one for '.' to watch for the appearance of the file and one for the file.
+        QCOMPARE(watch0.d->m_mapEntries.size(), initialSize + 2);
+    }
+    // NOTE: we leak the directory entry from above but it has no clients so it's mostly harmless
+    QCOMPARE(watch0.d->m_mapEntries.size(), initialSize + 1);
+}
+
+void KDirWatch_UnitTest::testMoveToThread()
+{
+    QTemporaryDir dir;
+    {
+        const QRegularExpression expression(QStringLiteral("KDirwatch is moving its thread. This is not supported at this time;.+"));
+        QTest::ignoreMessage(QtCriticalMsg, expression);
+
+        auto watch = new KDirWatch;
+        watch->addDir(dir.path());
+
+        auto thread = new QThread;
+        watch->moveToThread(thread);
+        thread->start();
+
+        waitUntilMTimeChange(dir.path());
+
+        QObject::connect(thread, &QThread::finished, thread, &QObject::deleteLater);
+        QObject::connect(thread, &QThread::finished, watch, &QObject::deleteLater);
+
+        thread->quit();
+        thread->wait();
+    }
+    // trigger an event on the now deleted watch. This should not crash!
+    const QString file = dir.path() + QLatin1String("/bar");
+    createFile(file);
+    waitUntilMTimeChange(file);
+}
+
 #include "kdirwatch_unittest.moc"
