@@ -18,18 +18,20 @@ static const int maxNameLength = _MAX_FNAME;
 static const int maxNameLength = NAME_MAX;
 #endif
 
-#include <QLatin1Char>
+#include "kcoreaddons_debug.h"
+#include "krandom.h"
 #include <QCoreApplication>
 #include <QDir>
+#include <QLatin1Char>
 #include <QLockFile>
 #include <QStandardPaths>
-#include "krandom.h"
-#include "kcoreaddons_debug.h"
 
 class KAutoSaveFilePrivate
 {
 public:
-    enum {NamePadding=8};
+    enum {
+        NamePadding = 8,
+    };
 
     QString tempFileName();
     QUrl managedFile;
@@ -42,13 +44,16 @@ static QStringList findAllStales(const QString &appName)
     const QStringList dirs = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
     QStringList files;
 
+    const QString suffix = QLatin1String("/stalefiles/") + appName;
     for (const QString &dir : dirs) {
-        QDir appDir(dir + QLatin1String("/stalefiles/") + appName);
-        //qCDebug(KCOREADDONS_DEBUG) << "Looking in" << appDir.absolutePath();
-        const auto listFiles = appDir.entryList(QDir::Files);
-        for (const QString &file : listFiles) {
-            files << (appDir.absolutePath() + QLatin1Char('/') + file);
+        QDir appDir(dir + suffix);
+        const QString absPath = appDir.absolutePath() + QLatin1Char('/');
+        qCDebug(KCOREADDONS_DEBUG) << "Looking in" << appDir.absolutePath();
+        QStringList listFiles = appDir.entryList(QDir::Files);
+        for (QString &file : listFiles) {
+            file.prepend(absPath);
         }
+        files += listFiles;
     }
     return files;
 }
@@ -70,32 +75,30 @@ QString KAutoSaveFilePrivate::tempFileName()
     // 7 for QLockFile's internal code (adding tmp .rmlock) = 16
     const int pathLengthLimit = maxNameLength - NamePadding - fileName.size() - protocol.size() - 16;
 
-    QString junk = KRandom::randomString(NamePadding);
+    const QString junk = KRandom::randomString(NamePadding);
     // This is done so that the separation between the filename and path can be determined
-    fileName += junk.rightRef(3) + protocol + QLatin1Char('_') + directory.leftRef(pathLengthLimit) + junk;
+    fileName += QStringView(junk).right(3) + protocol + QLatin1Char('_') + QStringView(directory).left(pathLengthLimit) + junk;
 
     return fileName;
 }
 
 KAutoSaveFile::KAutoSaveFile(const QUrl &filename, QObject *parent)
-    : QFile(parent),
-      d(new KAutoSaveFilePrivate)
+    : QFile(parent)
+    , d(new KAutoSaveFilePrivate)
 {
     setManagedFile(filename);
 }
 
 KAutoSaveFile::KAutoSaveFile(QObject *parent)
-    : QFile(parent),
-      d(new KAutoSaveFilePrivate)
+    : QFile(parent)
+    , d(new KAutoSaveFilePrivate)
 {
-
 }
 
 KAutoSaveFile::~KAutoSaveFile()
 {
     releaseLock();
     delete d->lock;
-    delete d;
 }
 
 QUrl KAutoSaveFile::managedFile() const
@@ -130,8 +133,8 @@ bool KAutoSaveFile::open(OpenMode openmode)
 
     QString tempFile;
     if (d->managedFileNameChanged) {
-        QString staleFilesDir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) +
-                                QLatin1String("/stalefiles/") + QCoreApplication::instance()->applicationName();
+        QString staleFilesDir = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1String("/stalefiles/")
+            + QCoreApplication::instance()->applicationName();
         if (!QDir().mkpath(staleFilesDir)) {
             return false;
         }
@@ -145,9 +148,7 @@ bool KAutoSaveFile::open(OpenMode openmode)
     setFileName(tempFile);
 
     if (QFile::open(openmode)) {
-
-        if (!d->lock)
-        {
+        if (!d->lock) {
             d->lock = new QLockFile(tempFile + QLatin1String(".lock"));
             d->lock->setStaleLockTime(60 * 1000); // HARDCODE, 1 minute
         }
@@ -155,7 +156,7 @@ bool KAutoSaveFile::open(OpenMode openmode)
         if (d->lock->isLocked() || d->lock->tryLock()) {
             return true;
         } else {
-            qCWarning(KCOREADDONS_DEBUG)<<"Could not lock file:"<<tempFile;
+            qCWarning(KCOREADDONS_DEBUG) << "Could not lock file:" << tempFile;
             close();
         }
     }
@@ -163,33 +164,35 @@ bool KAutoSaveFile::open(OpenMode openmode)
     return false;
 }
 
-static QUrl extractManagedFilePath(const QString& staleFileName)
+static QUrl extractManagedFilePath(const QString &staleFileName)
 {
+    const QStringView stale{staleFileName};
     // Warning, if we had a long path, it was truncated by tempFileName()
     // So in that case, extractManagedFilePath will return an incorrect truncated path for original source
-    const QStringRef sep = staleFileName.rightRef(3);
-    int sepPos = staleFileName.indexOf(sep);
-    const QByteArray managedFilename = staleFileName.leftRef(sepPos).toLatin1();
+    const auto sep = stale.right(3);
+    const int sepPos = staleFileName.indexOf(sep);
+    const QByteArray managedFilename = stale.left(sepPos).toLatin1();
 
     const int pathPos = staleFileName.indexOf(QChar::fromLatin1('_'), sepPos);
     QUrl managedFileName;
-    //name.setScheme(file.mid(sepPos + 3, pathPos - sep.size() - 3));
-    const QByteArray encodedPath = staleFileName.midRef(pathPos+1, staleFileName.length()-pathPos-1-KAutoSaveFilePrivate::NamePadding).toLatin1();
+    // name.setScheme(file.mid(sepPos + 3, pathPos - sep.size() - 3));
+    const QByteArray encodedPath = stale.mid(pathPos + 1, staleFileName.length() - pathPos - 1 - KAutoSaveFilePrivate::NamePadding).toLatin1();
     managedFileName.setPath(QUrl::fromPercentEncoding(encodedPath) + QLatin1Char('/') + QFileInfo(QUrl::fromPercentEncoding(managedFilename)).fileName());
     return managedFileName;
 }
 
-bool staleMatchesManaged(const QString& staleFileName, const QUrl &managedFile)
+bool staleMatchesManaged(const QString &staleFileName, const QUrl &managedFile)
 {
-    const QStringRef sep = staleFileName.rightRef(3);
+    const QStringView stale{staleFileName};
+    const auto sep = stale.right(3);
     int sepPos = staleFileName.indexOf(sep);
     // Check filenames first
-    if (managedFile.fileName() != QUrl::fromPercentEncoding(staleFileName.leftRef(sepPos).toLatin1())) {
+    if (managedFile.fileName() != QUrl::fromPercentEncoding(stale.left(sepPos).toLatin1())) {
         return false;
     }
     // Check paths
     const int pathPos = staleFileName.indexOf(QChar::fromLatin1('_'), sepPos);
-    const QByteArray encodedPath = staleFileName.midRef(pathPos + 1, staleFileName.length() - pathPos - 1 - KAutoSaveFilePrivate::NamePadding).toLatin1();
+    const QByteArray encodedPath = stale.mid(pathPos + 1, staleFileName.length() - pathPos - 1 - KAutoSaveFilePrivate::NamePadding).toLatin1();
     return QUrl::toPercentEncoding(managedFile.path()).startsWith(encodedPath);
 }
 
