@@ -5,7 +5,7 @@
     SPDX-FileCopyrightText: 2000 David Faure <faure@kde.org>
     SPDX-FileCopyrightText: 2006 Kevin Ottens <ervin@kde.org>
 
-    SPDX-License-Identifier: LGPL-2.0-only
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KJOB_P_H
@@ -15,9 +15,12 @@
 #include <QEventLoopLocker>
 #include <QMap>
 
+#include <array>
+
 class KJobUiDelegate;
 class QTimer;
 class QEventLoop;
+class QElapsedTimer;
 
 // This is a private class, but it's exported for
 // KIO::Job's usage. Other Job classes in kdelibs may
@@ -28,16 +31,28 @@ public:
     KJobPrivate();
     virtual ~KJobPrivate();
 
+    void speedTimeout();
+
     KJob *q_ptr = nullptr;
 
     KJobUiDelegate *uiDelegate = nullptr;
     QString errorText;
     int error = KJob::NoError;
     KJob::Unit progressUnit = KJob::Bytes;
-    QMap<KJob::Unit, qulonglong> processedAmount;
-    QMap<KJob::Unit, qulonglong> totalAmount;
+
+    struct Amounts {
+        qulonglong processedAmount = 0;
+        qulonglong totalAmount = 0;
+    };
+
+    std::array<Amounts, KJob::UnitsCount> m_jobAmounts;
+
     unsigned long percentage = 0;
     QTimer *speedTimer = nullptr;
+
+    std::unique_ptr<QElapsedTimer> elapsedTimer;
+    qint64 accumulatedElapsedTime = 0;
+
     QEventLoop *eventLoop = nullptr;
     // eventLoopLocker prevents QCoreApplication from exiting when the last
     // window is closed until the job has finished running
@@ -45,10 +60,9 @@ public:
     KJob::Capabilities capabilities = KJob::NoCapabilities;
     bool suspended = false;
     bool isAutoDelete = true;
-
-    void speedTimeout();
-
+    bool m_hideFinishedNotification = false;
     bool isFinished = false;
+    bool m_startedWithExec = false;
 
     Q_DECLARE_PUBLIC(KJob)
 };
